function TestRGraphChart(name, type, uri) {
    // vars
    this.name = name;
    this.offset = new Date().getTimezoneOffset()*60;
    this.data = new Array();
    this.points = 0;
    this.grid = 5;
    this.title = null;
    this.legend = null;
    this.chart = null;
    this.drawTime = 0;
    this.type = type;
    this.websocket = new WebSocket(uri);
    var self = this;
    this.websocket.onopen = function() { self.reset(); };
    this.websocket.onmessage = function(evt) { self.success(evt); }
    this.websocket.onclose = function() { };
}

TestRGraphChart.MAX_LENGTH = 7*24*3600;

TestRGraphChart.prototype.changeGraph = function(type) {
    this.data = new Array();
    this.points = 0;
    this.grid = 5;
    this.title = null;
    this.legend = null;
    this.chart = null;
    this.drawTime = 0;
    this.type = type;
    this.reset();
}

TestRGraphChart.prototype.onmousedown = function(e) {
    var coords = RGraph.getMouseXY(e);
    // indicator div like RGraph in Zoom
    var canvasXY = RGraph.getCanvasXY(this.chart.canvas);
    var areaDiv  = document.createElement('DIV');
    areaDiv.__canvas__ = this.chart.canvas;
    areaDiv.style.position = 'absolute';
    areaDiv.style.left = canvasXY[0] + coords[0] + 'px'
    areaDiv.style.top = canvasXY[1] + coords[1] + 'px'
    areaDiv.style.width = 0;
    areaDiv.style.height = 0;
    areaDiv.style.border = '1px solid black';
    areaDiv.style.cursor = 'crosshair';
    areaDiv.style.backgroundColor = 'rgba(220,220,220,0.3)';
    areaDiv.style.display = 'none';
    areaDiv.onmouseover = function (e) {
        setTimeout(function () {
            e.target.style.opacity = 0.8;
        }, 50);
        setTimeout(function () {
            e.target.style.opacity = 0.6;
        }, 100);
        setTimeout(function () {
            e.target.style.opacity = 0.4;
        }, 150);
        setTimeout(function () {
            e.target.style.opacity = 0.2;
        }, 200);
        setTimeout(function () {
            e.target.style.opacity = 0;
        }, 250);
        setTimeout(function () {
            e.target.style.display = 'none';
        }, 300);
        setTimeout(function () {
            e.target = null;
        }, 350);
    }
    document.body.appendChild(areaDiv);
    RGraph.Registry.Set('chart.zoomed.area.mousedown', coords);
    RGraph.Registry.Set('chart.zoomed.area.areadiv', areaDiv);
    RGraph.Registry.Set('chart.zoomed.area.time', new Date().getTime());
}


TestRGraphChart.prototype.onmousemove = function(e) {
    var startCoords = RGraph.Registry.Get('chart.zoomed.area.mousedown');
    if (startCoords && startCoords.length) {
        var coords      = RGraph.getMouseXY(e);
        //var canvasXY    = RGraph.getCanvasXY(e.target);
        var obj = e.target.__object__;
        //var context = obj.context;
        var canvas  = obj.canvas;
        var startx  = startCoords[0];
        var starty  = startCoords[1];
        var endx    = coords[0];
        var endy    = coords[1];
        var width   = endx - startx;
        var height  = endy - starty;
        var areaDiv = RGraph.Registry.Get('chart.zoomed.area.areadiv');
        if (width >= 0 && height >= 0) {
            if (width > 5 && height > 5) {
                areaDiv.style.width = (width - 5) + 'px';
                areaDiv.style.height = (height - 5) + 'px';
                areaDiv.style.display = 'inline';
            } else {
                areaDiv.style.display = 'none';
            }

        } else if (width < -5 || height < -5) {
            var canvasCoords = RGraph.getCanvasXY(canvas);
            var noticeDiv = document.createElement('DIV');
            noticeDiv.style.position = 'absolute';
            noticeDiv.style.top  = 0;
            noticeDiv.style.left = 0;
            noticeDiv.style.zIndex = 99;
            noticeDiv.style.border = '1px dashed black';
            noticeDiv.style.backgroundColor = '#ffc1c1';
            noticeDiv.style.MozBoxShadow = '0 0 5px #999';
            noticeDiv.style.WebkitBoxShadow = '0 0 5px #999';
            noticeDiv.style.boxShadow = '0 0 5px #999';
            noticeDiv.style.padding = '2px';
            noticeDiv.innerHTML = 'Zoom goes right and down';
            document.body.appendChild(noticeDiv);
            // Reposition the warning
            noticeDiv.style.top  = canvasCoords[1] + startCoords[1] - noticeDiv.offsetHeight + 'px';
            noticeDiv.style.left = canvasCoords[0] + startCoords[0] - (noticeDiv.offsetWidth / 2) + 'px';
            noticeDiv.style.width = noticeDiv.offsetWidth + 'px';
            setTimeout(function () {
                noticeDiv.style.opacity = 0.8;
            }, 2100);
            setTimeout(function () {
                noticeDiv.style.opacity = 0.6;
            }, 2200);
            setTimeout(function () {
                noticeDiv.style.opacity = 0.4;
            }, 2300);
            setTimeout(function () {
                noticeDiv.style.opacity = 0.2;
            }, 2400);
            setTimeout(function () {
                noticeDiv.style.display = 'none';
            }, 2500);
            setTimeout(function () {
                noticeDiv = null;
            }, 2600);
        }
    }
}

TestRGraphChart.prototype.onmouseup = function(e) {
    RGraph.FixEventObject(e);
    var startCoords = RGraph.Registry.Get('chart.zoomed.area.mousedown');
    var startTime = RGraph.Registry.Get('chart.zoomed.area.time');
    if (startCoords && startTime) {
        var coords = RGraph.getMouseXY(e);
        RGraph.Registry.Set('chart.zoomed.area.mousedown', null);
        RGraph.Registry.Set('chart.zoomed.area.time', null);
        RGraph.Registry.Get('chart.zoomed.area.areadiv').style.display = 'none';
        RGraph.Registry.Get('chart.zoomed.area.areadiv').style.left = '-1000px';
        RGraph.Registry.Get('chart.zoomed.area.areadiv').style.top = '-1000px';
        RGraph.Registry.Set('chart.zoomed.area.areadiv', null);
        var time = new Date().getTime();
        if (coords[0] < startCoords[0] + 5 || coords[1] < startCoords[1] + 5 || (time - startTime) < 200) {
            // incorrect select
            return;
        }
        startCoords[0] -= this.chart.Get('chart.gutter');
        coords[0] -= this.chart.Get('chart.gutter');
        // calculate startX and endX via cross multiplication
        var newStartX = Math.floor(this.startX + ((this.endX - this.startX) * startCoords[0]) / (this.chart.canvas.width - 2*this.chart.Get('chart.gutter')));
        var newEndX = Math.floor(this.startX + ((this.endX - this.startX) * coords[0]) / (this.chart.canvas.width - 2*this.chart.Get('chart.gutter')));
        this.startX = newStartX;
        this.endX = newEndX;
        this.calculateGridStep();
        this.callWS();
    }
}

TestRGraphChart.prototype.calculateGridStep = function()  {
    this.gridStep = (this.endX - this.startX) / this.grid;
    if (this.gridStep >= 24*3600) {
        this.gridStep = 24*3600*(Math.floor(this.gridStep / (24*3600)));
    } else if (this.gridStep > 9*3600) {
        this.gridStep = 12*3600;
    } else if (this.gridStep > 4.5*3600) {
        this.gridStep = 6*3600;
    } else if (this.gridStep > 2*3600) {
        this.gridStep = 3*3600;
    } else if (this.gridStep > 2700) {
        this.gridStep = 3600;
    } else if (this.gridStep > 1350) {
        this.gridStep = 1800;
    } else if (this.gridStep > 675) {
        this.gridStep = 900;
    } else if (this.gridStep > 450) {
        this.gridStep = 600;
    } else if (this.gridStep > 233) {
        this.gridStep = 300;
    } else if (this.gridStep > 90) {
        this.gridStep = 120;
    } else {
        this.gridStep = 60;
    }
}

TestRGraphChart.prototype.success = function(evt) {
    //alert("receiving... " + evt.data);
    var json = JSON.parse(evt.data);
    this.title = json.description;
    this.legend = json.legend;
    this.data = new Array();
    if (json.data != null) {
        var keys = Object.keys(json.data);
        for (this.points = 0; this.points < keys.length; this.points++) {
            this.data[this.points] = [
                Math.floor(keys[this.points].substr(1)/1000),
                json.data[keys[this.points]]
            ];
        }
    }
    this.createGraph();
}

TestRGraphChart.prototype.callWS = function() {
    var jsondata = {
        "type":this.type,
        "start":'T' + this.startX*1000,
        "end":'T' + this.endX*1000
    };
    //alert("sending... " + JSON.stringify(jsondata));
    this.websocket.send(JSON.stringify(jsondata));
}

TestRGraphChart.prototype.createGraph = function() {
    // calculate the new points
    var d = new Date();
    this.chart = new RGraph.Scatter(this.name, this.data);
    this.chart.Set('chart.gutter', 50);
    this.chart.Set('chart.tickmarks', 'square');
    this.chart.Set('chart.text.size', 6);
    this.chart.Set('chart.ticksize', 3);
    this.chart.Set('chart.xticks', false);
    //this.chart.Set('chart.background.barcolor1', 'white');
    //this.chart.Set('chart.background.barcolor2', 'white');
    this.chart.Set('chart.background.grid', true);
    this.chart.Set('chart.background.grid.autofit', true);
    this.chart.Set('chart.background.grid.autofit.numhlines', 10);
    this.chart.Set('chart.background.grid.autofit.numvlines', 10);
    this.chart.Set('chart.xaxispos', 'bottom');
    this.chart.Set('chart.title', this.title);
    this.chart.Set('chart.ylabels.count', 5);
    this.chart.Set('chart.title.xaxis', 'time');
    this.chart.Set('chart.title.yaxis', this.legend);
    this.chart.Set('chart.scale.decimals', 1);
    this.chart.Set('chart.title.yaxis.pos', 0.13);
    this.chart.Set('chart.xmin', this.startX);
    this.chart.Set('chart.xmax', this.endX);
    //this.chart.Set('chart.resizable', true);
    // calculate the labels
    var labels = new Array();
    var labelTime = this.startX;
    if ((this.startX % this.gridStep) > 0) {
        labelTime = labelTime - (this.startX % this.gridStep) + this.gridStep;
    }
    var i = 0;
    while (labelTime < this.endX) {
        var labelText = new Date((labelTime + this.offset)*1000).format('MM/dd/yyyy HH:mm');
        labels[i] = [labelText, labelTime];
        i++;
        labelTime += this.gridStep;
    }
    this.chart.Set('chart.labels', labels);
    //this.chart.Set('chart.text.angle', 90);
    //if (!RGraph.isIE8()) {
    //    this.chart.Set('chart.zoom.factor', this.zoom);
    //    this.chart.Set('chart.zoom.mode', 'area');
    //}
    RGraph.Clear(this.chart.canvas);
    this.chart.Draw();
    RGraph.Text(this.chart.context, 'Verdana', 8, 1, 20,
        "From " + new Date((this.startX + this.offset)*1000).format('MM/dd/yyyy HH:mm') + " to " +
        new Date((this.endX + this.offset)*1000).format('MM/dd/yyyy HH:mm') + " (GMT)");
    this.drawTime = new Date().getTime() - d.getTime();
    RGraph.Text(this.chart.context, 'Verdana', 8, 1, 33, 
        "Drawn " + this.points + " points in " + this.drawTime + "ms");
    var self = this;
    // add listeners to select via mouse
    this.chart.canvas.onmousedown = function(event){self.onmousedown(event);};
    this.chart.canvas.onmouseup = function(event){self.onmouseup(event);};
    this.chart.canvas.onmousemove = function(event){self.onmousemove(event);};
}

TestRGraphChart.prototype.forward = function() {
    this.startX += this.gridStep;
    this.endX += this.gridStep;
    this.calculateGridStep();
    this.callWS();
}

TestRGraphChart.prototype.reverse = function() {
    this.startX -= this.gridStep;
    this.endX -= this.gridStep;
    this.calculateGridStep();
    this.callWS();
}

TestRGraphChart.prototype.enlargeStart = function() {
    if (this.endX - this.startX < TestRGraphChart.MAX_LENGTH) {
        this.startX -= this.gridStep;
        this.calculateGridStep();
        this.callWS();
    }
}

TestRGraphChart.prototype.enlargeEnd = function() {
    if (this.endX - this.startX < TestRGraphChart.MAX_LENGTH) {
        this.endX += this.gridStep;
        this.calculateGridStep();
        this.callWS();
    }
}

TestRGraphChart.prototype.reset = function() {
    // set the complete day (today)
    this.startX = new Date().getTime();
    this.startX = (this.startX - (this.startX % (24*3600*1000)))/1000;
    this.endX = this.startX + 24*3600;
    this.calculateGridStep();
    // create the graph
    this.callWS();
}